/*
   calculation of ping statistics: calculations tested in excel pingfilter.xlsx
*/
bool pingStats() {
  char buff[120];
  //get values for ping statistics
  float avg_time_ms = Ping.averageTime();
  pingTime = long(1000 * avg_time_ms);  //all calcs below have times in MICROSECONDS to preserve precision in filter
  pingsLost = Ping.errors(); //pingsLost is type int
    
  if (pingsLost == 0) {
    //the line is UP so 
    lineDown = 0;  //reset the linedown counter to 0 - this handles short outages less than threshold
    countUp++;  //increment the up time
    // current values  
    snprintf(buff, 120, "pingStats: ping time is: %i ms;  last ping time was: %i ms; lost pings: %i", pingTime/1000, pingLast/1000, pingsLost);
    Serial.print(buff);
    
    //calculate average ping time
    if (pingAvg == 0) {
      pingAvg = pingTime; //if its the first value use it as a previous value also
    }
    pingAvg = (pingTime + (pingFilterM1 * pingAvg)) / pingFilter;
    pingAvgm = pingAvg / 1000;
    Serial.print("    ping average msec: ");
    Serial.println(pingAvgm);
    
  //calculate jitter
    diff = abs(pingTime - pingLast); 
    if(diff>1000)diff -= 1000; //V2.04: ignore a difference of 1 ms or less
    pingLast = pingTime;
    pingJitter = ((2 * diff)  + (qualFilterM1  * pingJitter)) / (qualFilter ); //add and average the jitter
    if (pingJitter > 99999) { pingJitter = 99999;   }
    pingQual = 100 - (pingJitter / 1000) ; //express as a value  between 100 & 0

    snprintf(buff, 120, "pingStats: difference is:  %i ms; jitter: %i ms;  pingQuality:  %4.0f %%", diff/1000, pingJitter/1000, pingQual);
    Serial.println(buff);
  }
  
  else {    // the web is DOWN: 
   lineDown += 1;
    if (lineDown >= threshold) { //we havent just missed a single ping so ..
      lineDown = threshold; //dont let it keep counting
      countDown++;
      if (countDown < samples){countDown = samples;} //so down time will always show even if less than 1 min
      countUp = 0; //reset up time
     }
    //web down so ping time and quality invalid
  }
  //calculate up and down times as dd:hh:mm
  minsUp = (countUp * timeInterval) / 60.0;
  hhUp = minsUp / 60;
  ddUp = hhUp / 24;
  hhUp = hhUp % 24;
  minsUp = minsUp % 60;

  minsDown = (countDown * timeInterval ) / 60.0;
  hhDown = minsDown / 60;
  minsDown = minsDown % 60;
  ddDown = hhDown / 24;
  hhDown = hhDown % 24;

  snprintf(buff, 120, "pingStats:  %2i:%2i:%2i: < Up : Down > %2i:%2i:%2i: \n",ddUp,hhUp,minsUp,ddDown,hhDown,minsDown);
  Serial.println(buff);
  return true;
}

void showStats() {
  snprintf(stUp,  sizeof(stUp), "up %2.1id%2.2ih%2.2im",ddUp,hhUp,minsUp);
  snprintf(stDown,  sizeof(stDown), "dn %2.1id%2.2ih%2.2im",ddDown,hhDown,minsDown); //2.1i shows days as max 2 min 1 character
  snprintf(spAvg,  sizeof(spAvg), "ping:  %4.1i ms", pingAvgm);
  snprintf(sqAvg,  sizeof(sqAvg), "quality: %4.0f %%", pingQual);  //pingQual is type float

  tft.fillScreen(TFT_BLACK);
  tft.setTextColor(TFT_GREEN, TFT_BLACK);

  //poX=0 is too close to left margin. poY=0 is too close to top margin;  1 is fine
  tft.drawString(sTitle, 1, 1, 4);  //title
  tft.drawString(stUp, 1, 27, 4);  //uptime
  tft.drawString(stDown, 1, 53, 4);  //downtime
  if (lineDown >= threshold) {
    tft.setTextColor(TFT_BLUE, TFT_BLACK);
  }
  else {
    tft.setTextColor(TFT_GREEN, TFT_BLACK);
  }
  tft.drawString(spAvg, 1, 79, 4); //ping
  tft.drawString(sqAvg, 1, 105, 4); //quality
  showDone = true;
}
