
/*
   Now using ESP32.
   Ping working well giving sensible values (20ms for talktalk) with possible long delays
   doing 2 pings at a time with a threshold of 3 attempts to record an outage
   2.04 changes: bool buttonState, lastButtonState, switchState, lastSwitchState; changed from type int
   2.05 May 2023 following a fail of the test server change to ping Quad9 as a more reliable DNS server
*/

#include <TFT_eSPI.h> // Graphics and font library for ST7735 driver chip
#include <SPI.h>      //supporting spi interface with display
#include "creds.h"            // Replace with your network credentials
#include "webdisp.h"          //variable declarations
#include <WiFi.h>             //WiFi functions
#include <ESP32Ping.h>        //handles the ping functions   https://github.com/marian-craciunescu/ESP32Ping
#include <time.h>             //support for time functions

// Set global to avoid object removing after setup() routine
TFT_eSPI tft = TFT_eSPI();  // Invoke library, pins defined in User_Setup.h (see notes)

time_t now;                         // this is the epoch
tm tm;

//const IPAddress remote_ip(192, 168, 1, 1);  //ping the router
//const IPAddress remote_ip(192, 168, 1, 22);  //to ping network printer
//const IPAddress remote_ip(185, 217, 104, 171); //to ping TheHUT: doesnt work if in header file
const IPAddress remote_ip(9,9,9,9); //to ping Quad9

void connectWiFi();//in wifi tab
void printLocalTime(); //in wifi tab
void checkSwitches(); //in chkSws tab
bool pingStats();//in pingstats tab
void showStats(); //in pingstats tab


void setup() {
  tft.init();
  tft.setRotation(1);
  tft.fillScreen(TFT_BLACK);
  // Begin serial connection
  Serial.begin(115200);
  // set up button and switch inputs & default states
  pinMode(buttonPin, INPUT_PULLUP);
  pinMode(switchPin, INPUT_PULLUP);
  buttonState = 1;
  lastSwitchState = !digitalRead(switchPin); //2.04: ensure it will be processed by checkSwitches(); at start up

  samples = 60 / timeInterval; //used in pingstats for checking down time

  delay(200); // serial comms is disturbed during a restart
  Serial.print("Connecting to ");
  Serial.println(ssid);

  //monitor Wifi connection and handle D0   WiFi indicator
  connectWiFi();
  configTime(0, 0, MY_NTP_SERVER); // --> Here is the IMPORTANT ONE LINER needed in your sketch!

  delay(10000); //allow to get time
  printLocalTime();
  showDone = false;  //prepare to update display
  timeWas = millis();
}

void loop() {
  checkSwitches();
  timeNow = millis();
  if (timeNow >= (timeWas + (timeInterval * 1000))) {  //time for another ping?
    timeWas = timeNow;
    Serial.print("Pinging ip  ");
    Serial.println(remote_ip);
    //ping it
    bool pingOK = Ping.ping(remote_ip, pingNumber);  //produces values for pingTime, expectedCount, success, errors
      pingStats();
      showDone = false;  //prepare to update display
      showStats();
  }
}
